/*******************************************************************************
* Copyright (c) 2006 IONA Technologies PLC
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
* 
* Contributors:
*     IONA Technologies PLC - initial API and implementation
*******************************************************************************/
package org.eclipse.stp.sc.jaxws.builders;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceStatus;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.stp.common.logging.LoggingProxy;
import org.eclipse.stp.sc.common.builders.ScJavaBuilder;
import org.eclipse.stp.sc.common.utils.JDTUtils;
import org.eclipse.stp.sc.common.workspace.WorkspaceManager;
import org.eclipse.stp.sc.jaxws.ScJaxWsPlugin;
import org.eclipse.stp.sc.jaxws.runtimeprovider.IJavaToWsdlGenerator;
import org.eclipse.stp.sc.jaxws.workspace.JaxWsWorkspaceManager;

public class ScJavaToWsdlBuilder extends ScJavaBuilder {

    public static final String BUILDER_ID = "org.eclipse.stp.sc.jaxws.builders.javatowsdlbuilder";
    
    private static final LoggingProxy LOG = LoggingProxy.getlogger(ScJavaToWsdlBuilder.class);
    
    public ScJavaToWsdlBuilder() {
        super();
        LOG.debug("JavaToWSDL builder has been created");
    }
    
    /**
     * call the javatowsdl to build one JWS style java source file
     * @param javaFile, the java file with JWS annotation
     */
    protected void buildOneJavaFile(IFile javaFile) throws CoreException {
        try {
            if (javaFile.getFileExtension() == null
                || !javaFile.getFileExtension().equals("java")) {
                return;
            }
            generate(javaFile.getFullPath(), getProject(), null);
        } catch (Exception e) {
            LOG.debug("generation failure", e);

            Status status = new Status(IStatus.ERROR,
                                       ResourcesPlugin.PI_RESOURCES,
                                       IResourceStatus.BUILD_FAILED,
                                       e.getMessage(),
                                       e);
            throw new CoreException(status);
        }
    }
    
    private void generate(IPath path, IProject project, Object data) throws CoreException {

    	boolean isAutoBuild = WorkspaceManager.getWorkspaceAutoBuild();
		try {
			if (isAutoBuild) {
		    	// disable auto-build first before call generator
				LOG.debug("disable workspace autobuild before generating code");
				WorkspaceManager.setWorkspaceAutoBuild(false);
			}
			IJavaToWsdlGenerator generator = ScJaxWsPlugin
					.getRuntimeProviderManager()
					.getJavaToWsdlGenerator(project);
			if (data != null) {
				generator.setInitializationData(null, null, data);
			}
			generator.run(path, project);
		} finally {
			// enable auto-build again after call generator
			if (isAutoBuild) {
				LOG.debug("re-enable workspace autobuild after generating code");
				WorkspaceManager.setWorkspaceAutoBuild(true);
			}
		}
	}

	@Override
	protected boolean checkAnnotation(IFile javaFile) {
		return JavaDocumentUtils.hasWebServiceAnnotation(javaFile);
	}
	
	/**
     * clean the generated wsdl. This will delete /project/wsdl dir and all contains under that dir
     * @param monitor, the progress monitor
     * @throws CoreException
     */
    protected void clean(IProgressMonitor monitor) throws CoreException {
		LOG.debug("clean for the project:" + getProject().getName());
		IContainer wsdlFolder = JaxWsWorkspaceManager.getWSDLFolder(getProject());
		if(wsdlFolder instanceof IFolder){
			wsdlFolder.delete(true, monitor);
		}
		else if(wsdlFolder instanceof IProject){
			IResource[] resources = ((IProject)wsdlFolder).members();
			for(int i = 0; i < resources.length; i++){
				if("wsdl".equals(resources[i].getFileExtension())){
					resources[i].delete(true, monitor);
				}
			}
		}
		getProject().refreshLocal(IProject.DEPTH_INFINITE, null);
	}

	protected void removeResourceFile(IResource res) throws CoreException {
		if (!(res instanceof IFile)) {
			return;
		}

		IFile file = (IFile) res;

		if (!file.getFileExtension().equals("java")) {
			return;
		}

		if (checkAnnotation(file)) {
			LOG.debug("WebService java file has been removed:"
					+ file.getFullPath());

			String clsName = JDTUtils.getJavaClassNameFromFile(file);

			if (clsName.lastIndexOf(".") > 0) {
				clsName = clsName.substring(clsName.lastIndexOf("."));
			}

			String wsdlName = clsName + ".wsdl";
			LOG.debug("  need to remove:" + wsdlName);

			IContainer folder = JaxWsWorkspaceManager.getWSDLFolder(getProject());
			IResource wsdlResource = folder.findMember(wsdlName);

			if (wsdlResource.exists()) {
				LOG.debug("delete wsdl resource:" + wsdlResource.getFullPath());
				wsdlResource.delete(IResource.FORCE, null);
			}
		}
	}
    
    
}
